// Multi rom cartridge selector
// for the VECTREX game system
// By A.Leverett 2021 (coronavirus pandemic era)
//
// RomMax should be set to the number of roms on the eprom.
// if you have 5 rom images on the eprom then this no. should be 5
// if you have 128 then this no. should be 128
// remember this program is for both 'DIY Cart+8' and 'DIY Cart+32'
// 'DIY Cart+8' is for 8KB images and 'DIY Cart+32' is for 32KB images.
// 8KB images will work on the +32 version as they only take up 8KB of the 32KB available
// 32KB images WILL NOT WORK on the +8 version
//
// compatible with these ( and pin compatible ) eproms and how many images they will store.
// eprom    8KB   32KB
// 27C010   16    4
// 27C020   32    8
// 27C040   64    16
// 27C080   128   32
//
// Save routine added. If you hold select until the display shows 'Saved!' then when you
// next power up the system with the cartridge in , the saved cartridge number will be selected 
// automatically!
//
//----------------------------------------------------------------------------------------


#include "SSD1306Ascii.h"
#include "SSD1306AsciiAvrI2c.h"
#include "EEPROM.h"
#define I2C_ADDRESS 0x3C     // 0X3C+SA0 - 0x3C or 0x3D
#define dpin 8               //'-1'button
#define upin 10              //'+1'button
#define spin 9               //'select' button
#define res 13               // reset pin ( only works from cart list program (rom 01))
 
int Romnum = 1;
int sel = 0;
int RomMax = 35;      // number of cartridges on rom
int tst = 1;
int selst = 1;
int ust = 1;
int romsave;
int sav = 0;

SSD1306AsciiAvrI2c oled;

//----------------------------------------------------------------------------------------

void setup() {
  EEPROM.get(0 , romsave);           //load saved rom no.
  if (romsave != (Romnum -1))
  {
    Romnum = (romsave + 1);
  }

  pinMode(dpin , INPUT_PULLUP);
  pinMode(upin , INPUT_PULLUP);
  pinMode(spin , INPUT_PULLUP);
  pinMode(res , OUTPUT);
  digitalWrite(res, LOW);
  DDRD = B11111111;
  PORTD = (Romnum - 1);              //write initial rom no.

  oled.begin(&Adafruit128x32, I2C_ADDRESS);
  oled.setFont(Adafruit5x7);
  oled.clear();
  oled.set2X();
  oled.println("  VECTREX   ");
  oled.println(" DIY CART+  ");
  delay(6000);
  oled.clear();
}
  
//----------------------------------------------------------------------------------------

void loop() {

  //set display
 
  // first row
  oled.setCursor(0 , 0);
  oled.set2X();
  if (Romnum <= 9)
  {
  oled.print("ROM 0");
  }
  else 
  { 
  oled.print("ROM ");
  }
  oled.print (Romnum);
  oled.print("       ");
  oled.println();

  // second row
  oled.set1X();
  oled.println();
  
  //third row
  oled.print("Down    Select     Up     ");

  // read switches
  tst = digitalRead(dpin);
  if (tst == 0)
  {
  Romnum = (Romnum - 1);
  sav=0;
    if (Romnum == 0)
    {
    Romnum = RomMax;  
    }
  } 
  ust = digitalRead(upin);
  if (ust == 0)
  {
  Romnum = (Romnum + 1);
  sav=0;  
    if (Romnum > RomMax)
    {
    Romnum = 1;
    }
  }
  selst = digitalRead(spin);
  if (selst == 0)
  {
    digitalWrite(res , HIGH);
    delay (25);
    digitalWrite(res , LOW);
    PORTD = (Romnum - 1);
    sav = (sav + 1);
    oled.setCursor(0,0);
    oled.set2X();
    oled.clear();
    oled.println ("  SELECTED  ");
    delay(400);
    if (sav == 4)
    {
      EEPROM.put(0 , (Romnum -1));    //save rom no.
      sav = 0;
      oled.setCursor(0,0);
      oled.set2X();
      oled.clear();
      oled.println ("   SAVED!  ");
      delay(800);
    }

  }
  
   
  delay(400);
  }

  // end of code.
